//custom.cc: Implementation file for useful custom circuits

#include "signal.h"
#include "custom.h"

signal_type Wired_OR2(signal_type inp0, signal_type inp1)
{
	//implemented as a wire gate (for T-gate muxes)
	signal_type output;
	switch(inp0)
		{
		case True:
			switch(inp1)
				{
				case True:
					output = True;
					break;
				case False:
					output = Unknown;
					break;
				case High_Impedance:
					output = True;
					break;
				default:
					output = Unknown;
				} //switch
			break;
		case False:
			switch(inp1)
				{
				case True:
					output = Unknown;
					break;
				case False:
					output = False;
					break;
				case High_Impedance:
					output = False;
					break;
				default:
					output = Unknown;
				} //switch
			break;
		case High_Impedance:
			switch(inp1)
				{
				case True:
					output = True;
					break;
				case False:
					output = False;
					break;
				case High_Impedance:
					output = Unknown;
					break;
				default:
					output = Unknown;
				} //switch
			break;
		default:
			output = Unknown;
		} //switch
	return output;
} //Wired_OR2


signal_type TINV(signal_type data, signal_type control)
{
//three state inverter
	signal_type output;
	switch(control)
		{
		case True:
			switch(data)
				{
				case True:
					output = False;
					break;
				case False:
					output = True;
					break;
				default:
					output = Unknown;
				} //switch
			break;
		case False:
			output = High_Impedance;
			break;
		default:
			output = Unknown;
		} //switch
	return output;
} //TINV


signal_type TRANS(signal_type input, signal_type control)
{
//transmission gate
	signal_type output;
	switch(control)
		{
		case True:
			switch(input)
				{
				case True:
				case False:
					output = input;
					break;
				default:
					output = Unknown;
				} //switch
			break;
		case False:
			output = High_Impedance;
			break;
		default:
			output = Unknown;
		} //switch
	return output;
} //TRANS


signal_type Wired_ORN(signal_type* input, unsigned int number_of_inputs)
{
//handles an arbitrary number of inputs
//if any input is True, the output will be True if nothing is Unknown or False
//if any input is False, the output will be False if nothing is True or Unknown
//if any input is Unknown, or all inputs are High_Impedance, the output is Unknown
//Ideally, one input is driven while all else is High_Impedance
unsigned int index;
unsigned int Trues_found = 0;
unsigned int Unknowns_found = 0;
unsigned int Falses_found = 0;
signal_type output = Unknown;

for(index = 0; index < number_of_inputs; index++)
	{
	switch(input[index])
		{
		case True:
			Trues_found++;
			break;
		case False:
			Falses_found++;
			break;
		case High_Impedance:
			break;
		default:
			Unknowns_found++;
		} //switch
	} //for
	//now we know what we have for inputs
	if((Trues_found > 0)&&(Falses_found == 0)&&(Unknowns_found == 0))
		{
			output = True;
		} //if
	else
		{
		if((Falses_found > 0)&&(Trues_found == 0)&&(Unknowns_found == 0))
			{
				output = False;
			} //if
		} //else
	return output;
} //Wired_ORN

//analog

signal_type Diff_amp(optical_type input, optical_type reference)
{
  //amplifies difference between two signals to binary
  //unstable when both inputs are equal

  signal_type output = Unknown; //Unknown by default
  if(input>=reference) //known to be at or above required voltage
    output = True;
  else
    if((input<reference)&&(reference != Undefined)) //known to be not enough
      output = False;
    else
      if(input == UH) //maximum input
	output = True; //even if reference is undefined, it will trigger
  return output;
} //Diff_amp

optical_type Voltage_Reference(signal_type* inputs)
{
  //first bit is most significant
  short int octal_value = 0;
  optical_type output = Undefined;
  bool Unknown_found = false;

  if(inputs[0] == True)
    octal_value += 4;
  else
    if(inputs[0] != False)
      Unknown_found = true;
  if(inputs[1] == True)
    octal_value += 2;
  else
    if(inputs[1] != False)
      Unknown_found = true;
  if(inputs[2] == True)
    octal_value += 1;
  else
    if(inputs[2] != False)
      Unknown_found = true;

  if(Unknown_found == false)
    {
      switch(octal_value)
	{
	case 0:
	  output = UL;
	  break;
	case 1:
	  output = VL;
	  break;
	case 2:
	  output = ML;
	  break;
	case 3:
	  output = L;
	  break;
	case 4:
	  output = H;
	  break;
	case 5:
	  output = MH;
	  break;
	case 6:
	  output = VH;
	  break;
	case 7:
	  output = UH;
	  break;
	default:
	  ;
	} //switch
    } //real output generated

  return output;
} //Reference 

///////////////////////////////////////////////////////////////
//End of implementation file





























